@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.bf.ml
 ## -- Module  : basics.py
 ## -------------------------------------------------------------------------------------------------
@@ -63,23 +63,59 @@
 ## -- 2023-03-09  2.1.1     DA       Class TrainingResults: removed parameter p_path
 ## -- 2023-03-10  2.1.2     DA       Class AdaptiveFunction: refactoring constructor parameters
 ## -- 2023-03-10  2.1.3     SY       Refactoring
+## -- 2022-03-16  2.1.4     SY       Add _get_accuracy(), add_objective, _add_objective in Model
 ## -- 2023-03-29  2.2.0     DA       Classes Model, Scenario: refactoring of persistence
+## -- 2023-04-10  2.2.1     SY       Update get_accuracy, add_objective in Model
+## -- 2023-07-24  2.3.0     LSB      New methods in context of hyperparameter tuning
+##                                   - _update_hyperparameters()
+##                                   - _hpt_updated
+##                                   - _hyperparameter_handler()
+## -- 2025-05-28  2.4.0     DA       - New class Adaptation
+## --                                - Class Model:
+## --                                  - Method _set_adapted(): new parameters
+## --                                  - Incorporation of new class Adaptation
+## -- 2025-06-02  2.5.0     DA       New class AdaptationType
+## -- 2025-07-15  2.5.1     DA       Class AdaptationType: replaced parent class StrEnum by str
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 2.2.0 (2023-03-29)
+Ver. 2.5.1 (2025-07-15)
 
 This module provides the fundamental templates and processes for machine learning in MLPro.
 
 """
 
+import random
+from datetime import datetime
+import os
 
 from mlpro.bf.various import *
+from mlpro.bf.events import Event
+from mlpro.bf.exceptions import *
+from mlpro.bf.plot import PlotSettings 
 from mlpro.bf.math import *
 from mlpro.bf.data import Buffer
 from mlpro.bf.mt import *
 from mlpro.bf.ops import Mode, ScenarioBase
-import random
+
+class Figure: pass
+
+
+
+# Export list for public API
+__all__ = [ 'Model',
+            'AWorkflow',
+            'Adaptation',
+            'AdaptationType',
+            'HyperParam',
+            'HyperParamSpace',
+            'HyperParamTuple',
+            'HyperParamDispatcher',
+            'Scenario',
+            'Training',
+            'TrainingResults',
+            'HyperParamTuner',
+            'AdaptiveFunction' ]
 
 
 
@@ -176,6 +212,55 @@ class HyperParamDispatcher (HyperParamTuple):
 
 
 
+## -------------------------------------------------------------------------------------------------
+## -------------------------------------------------------------------------------------------------
+class AdaptationType (str): #(StrEnum):
+    """
+    Basic types of adaptation.
+    """
+    NONE        = ''
+    FORWARD     = 'Forward'
+    EVENT       = 'Event'
+
+
+
+
+
+## -------------------------------------------------------------------------------------------------
+## -------------------------------------------------------------------------------------------------
+class Adaptation (Event):
+    """
+    Class for adaptation events raised by the Model class.
+
+    Parameters
+    ----------
+    p_raising_object
+        Raising object.
+    p_subtype : AdaptationType
+        Type of adaptation.
+    p_tstamp : TStampType = None
+        Time stamp of adaptation.
+    **p_kwargs
+        Further keyword arguments to be transported by the event.
+    """
+
+## -------------------------------------------------------------------------------------------------
+    def __init__( self, 
+                  p_raising_object, 
+                  p_subtype : AdaptationType,
+                  p_tstamp : TStampType = None, 
+                  **p_kwargs ):
+        
+        super().__init__( p_raising_object = p_raising_object, 
+                          p_tstamp = p_tstamp, 
+                          **p_kwargs )
+        
+        self.subtype = p_subtype
+
+
+
+
+
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
 class Model (Task, ScientificObject):
@@ -215,14 +300,15 @@ class Model (Task, ScientificObject):
         Further model specific hyperparameters (to be defined in child class).
     """
 
-    C_TYPE              = 'Model'
-    C_NAME              = '????'
+    C_TYPE                      = 'Model'
+    C_NAME                      = '????'
 
-    C_EVENT_ADAPTED     = 'ADAPTED'
+    C_EVENT_ADAPTED             = 'ADAPTED'  
+    C_EVENT_CLS                 = Adaptation  
 
-    C_BUFFER_CLS        = Buffer       
+    C_BUFFER_CLS                = Buffer  
 
-    C_SCIREF_TYPE       = ScientificObject.C_SCIREF_TYPE_NONE     
+    C_SCIREF_TYPE               = ScientificObject.C_SCIREF_TYPE_NONE
 
 ## -------------------------------------------------------------------------------------------------
     def __init__( self, 
@@ -252,6 +338,11 @@ class Model (Task, ScientificObject):
         self._hyperparam_tuple  = None
         self._init_hyperparam(**p_par)
 
+        for hyper_param in self._hyperparam_space.get_dims():
+            hyper_param.register_event_handler(p_event_id=HyperParam.C_EVENT_VALUE_CHANGED, p_event_handler=self._hyperparam_handler)
+
+        self._hp_latest = True
+
         if p_buffer_size > 0:
             self._buffer = self.C_BUFFER_CLS(p_size=p_buffer_size)
         else:
@@ -285,6 +376,32 @@ class Model (Task, ScientificObject):
         return self._hyperparam_tuple
 
 
+## -------------------------------------------------------------------------------------------------
+    def _hyperparam_handler(self, p_event_id, p_event_object:Event):
+        """
+        This is an event handler method for managing the updates of hyperparameters in the HPTuple.
+        Set's the flag hp_latest to false, as the hpt of the model are not latest to the tuple.
+        """
+        if p_event_id == HyperParam.C_EVENT_VALUE_CHANGED:
+            self._hp_latest = False
+
+
+## -------------------------------------------------------------------------------------------------
+    def _update_hyperparameters(self) -> bool:
+        """
+        Custom method to update the hyperparameters of a system with the latest value in the Hyperparameter Tuple.
+        This may be due to Hyperparameter Tuning. Please return True if the hyperparameters are updated successfully.
+
+        Returns
+        -------
+        bool:
+            True if the hyperparameters are updated successfully.
+
+        """
+
+        raise NotImplementedError
+
+
 ## -------------------------------------------------------------------------------------------------
     def switch_adaptivity(self, p_ada:bool):
         """
@@ -323,14 +440,34 @@ class Model (Task, ScientificObject):
 
 
 ## -------------------------------------------------------------------------------------------------
-    def _set_adapted(self, p_adapted:bool):
+    def _set_adapted( self, 
+                      p_adapted : bool,
+                      p_subtype : AdaptationType = AdaptationType.NONE,
+                      p_tstamp : TStampType = None,
+                      **p_kwargs ):
         """
-        Sets the adapted flag. Internal use only.
+        Sets the adapted flag and raises an adaptation event.
+
+        Parameters
+        ----------
+        p_adapted : bool
+            Adaptation flag. If True and event handlers are registered an adaptation event is raised.
+        p_subtype : AdaptationType
+            Subtype of adaptation. See class AdaptationType for further details.
+        p_tstamp : TStampType = None
+            Optional explicite time stamp.
+        **p_kwargs
+            Optional keyword arguments
         """
 
         self._adapted = p_adapted
 
-        if self._adapted: self._raise_event(self.C_EVENT_ADAPTED, Event(p_raising_object=self))
+        if ( self._adapted ) and ( len(self._registered_handlers) > 0 ): 
+            self._raise_event( p_event_id = self.C_EVENT_ADAPTED, 
+                               p_event_object = self.C_EVENT_CLS( p_raising_object = self,
+                                                                  p_subtype = p_subtype,
+                                                                  p_tstamp = p_tstamp,
+                                                                  **p_kwargs ) )
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -340,7 +477,7 @@ class Model (Task, ScientificObject):
 
         Parameters
         ----------
-        p_kwargs : dict
+        **p_kwargs
             All parameters that are needed for the adaption. Depends on the specific higher context.
 
         Returns
@@ -349,11 +486,23 @@ class Model (Task, ScientificObject):
             True, if something has been adapted. False otherwise.
 
         """
+        # Check if the hyperparameters are updated
+
+        if not self._hp_latest:
+            try:
+                self._hp_latest = self._update_hyperparameters()
+            except:
+                if not self._hp_latest:
+                    self.log(Log.C_LOG_TYPE_E, "No hyperparameter update mechanism provided. Runs will continue without update."
+                                  " Please read the documentation for method _update_hyperparameter() on Model class.")
 
         if not self._adaptivity: return False
         self.log(self.C_LOG_TYPE_S, 'Adaptation started')
         adapted = self._adapt(**p_kwargs)
-        self._set_adapted(adapted)
+
+        self._set_adapted( p_adapted = adapted,
+                           p_subtype = AdaptationType.FORWARD )
+        
         return adapted
         
 
@@ -394,7 +543,9 @@ class Model (Task, ScientificObject):
             Object with further context informations about the event.
         """
 
-        self._set_adapted(p_adapted=self._adapt_on_event(p_event_id=p_event_id, p_event_object=p_event_object))        
+        self._set_adapted( p_adapted = self._adapt_on_event( p_event_id=p_event_id, 
+                                                             p_event_object=p_event_object ),
+                           p_subtype = AdaptationType.EVENT )        
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -427,6 +578,24 @@ class Model (Task, ScientificObject):
         if self._buffer is not None: self._buffer.clear()
 
 
+## -------------------------------------------------------------------------------------------------
+    def add_objective(self, **p_kwargs):
+        """
+        Determines the objective of the model.
+        """
+
+        self._add_objective(**p_kwargs)
+
+
+## -------------------------------------------------------------------------------------------------
+    def _add_objective(self, **p_kwargs):
+        """
+        This method is called in add_objective(). PLease redefine this method.
+        """
+
+        raise NotImplementedError
+
+
 ## -------------------------------------------------------------------------------------------------
     def get_accuracy(self) -> float:
         """
@@ -438,6 +607,20 @@ class Model (Task, ScientificObject):
             Accuracy of the model as a scalar value in interval [0,1]
         """
 
+        return self._get_accuracy()
+
+
+## -------------------------------------------------------------------------------------------------
+    def _get_accuracy(self) -> float:
+        """
+        This method is called in get_accuracy(). PLease redefine this method.
+
+        Returns
+        -------
+        accuracy : float
+            Accuracy of the model as a scalar value in interval [0,1]
+        """
+
         raise NotImplementedError
 
 
@@ -1387,4 +1570,4 @@ class AdaptiveFunction (Function, Model):
                         p_class_shared = p_class_shared,
                         p_visualize = p_visualize,
                         p_logging = p_logging, 
-                        **p_par )
\ No newline at end of file
+                        **p_par )
