@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.bf
 ## -- Module  : mt
 ## -------------------------------------------------------------------------------------------------
@@ -32,10 +32,23 @@
 ## -- 2023-01-01  1.8.0     DA       Refactoring of plot settings
 ## -- 2023-02-15  1.8.1     DA       Class Task: changed default range to C_RANGE_THREAD
 ## -- 2023-03-27  1.9.0     DA       Class Task: added parent class Persistent
+## -- 2024-01-05  1.9.1     DA       Class Task: bugfix in __init__() regarding name generation
+## -- 2024-05-31  1.9.2     DA       Class Task: new exception rule for MacOs in meth. init_plot()
+## -- 2024-06-17  2.0.0     DA       Class Workflow: new method get_tasks()
+## -- 2024-06-18  2.1.0     DA       Class Task: new parent class KWArgs
+## -- 2024-10-07  2.2.0     DA       Classes Task, Workflow: new method reset()
+## -- 2024-11-10  2.2.0     DA       Refactoring of class Workflow regarding plotting
+## -- 2024-11-11  2.3.0     DA       Class Task:
+## --                                - new method _on_finished()
+## --                                - redefinition of method _raise_event()
+## -- 2024-12-10  2.3.1     DA       - Method Task.init_plot(): refactoring
+## --                                - Method Workflow.init_plot(): Bugfix and optimization
+## -- 2024-12-11  2.4.0     DA       New method Workflow.remove_plot()
+## -- 2025-07-18  2.5.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.9.0 (2023-03-27)
+Ver. 2.5.0 (2025-07-18)
 
 This module provides classes for multitasking with optional interprocess communication (IPC) based
 on shared objects. Multitasking in MLPro combines multrithreading and multiprocessing and simplifies
@@ -49,13 +62,15 @@ See also: https://stackoverflow.com/questions/40234771/replace-pickle-in-python-
 """
 
 
-from time import sleep
-import uuid
 import threading as mt
 import multiprocess as mp
-import matplotlib
-from matplotlib.figure import Figure
 from multiprocess.managers import BaseManager
+
+try:
+    from matplotlib.figure import Figure
+except:
+    class Figure : pass
+
 from mlpro.bf.exceptions import *
 from mlpro.bf.various import *
 from mlpro.bf.events import EventManager, Event
@@ -63,6 +78,14 @@ from mlpro.bf.plot import PlotSettings, Plottable
 
 
 
+# Export list for public API
+__all__ = [ 'Range',
+            'Shared',
+            'Async',
+            'Task',
+            'Workflow' ]
+
+
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -472,7 +495,7 @@ class Async (Range, Log):
 
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
-class Task (Async, EventManager, Plottable, Persistent): 
+class Task (Async, EventManager, Plottable, Persistent, KWArgs): 
     """
     Template class for a task, that can run things - and even itself - asynchronously in a thread
     or process. Tasks can run standalone or as part of a workflow (see class Workflow). The integrated
@@ -498,7 +521,7 @@ class Task (Async, EventManager, Plottable, Persistent):
     p_logging
         Log level (see constants of class Log). Default: Log.C_LOG_ALL
     p_kwargs : dict
-        Further optional named parameters.
+        Further optional keyword arguments.
     """
 
     C_TYPE              = 'Task'
@@ -520,26 +543,26 @@ class Task (Async, EventManager, Plottable, Persistent):
                   p_logging = Log.C_LOG_ALL,
                   **p_kwargs ):
 
-        self._kwargs            = p_kwargs.copy()
+        KWArgs.__init__(self, **p_kwargs)
 
         self._predecessor_tasks = []
         self._predecessor_ids   = []
         self._num_predecessors  = 0
         self._ctr_predecessors  = 0
 
+        Id.__init__(self, p_id=p_id)
+
+        if p_name is not None:
+            self.set_name(p_name)
+        else:
+            self.set_name(str(self.get_id()))
+            
         Async.__init__(self, p_range_max=p_range_max, p_class_shared=p_class_shared, p_logging=p_logging)
         EventManager.__init__(self, p_logging=p_logging)
         Plottable.__init__(self, p_visualize=p_visualize)
         Persistent.__init__(self, p_id=p_id, p_logging=p_logging)
 
         self._custom_run_method = self._get_custom_run_method()
-        name = self.get_name()
-        if name != '': name = name + ' '
-        if p_name is not None:
-            self.set_name(name + p_name)
-        else:
-            self.set_name(name + str(self.get_id()))
-
         self._autorun(p_autorun=p_autorun, p_kwargs=self._kwargs)
 
 
@@ -555,6 +578,11 @@ class Task (Async, EventManager, Plottable, Persistent):
         """
 
         return self.get_id()
+    
+
+## -------------------------------------------------------------------------------------------------
+    def reset(self, **p_kwargs):
+        pass
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -701,6 +729,20 @@ class Task (Async, EventManager, Plottable, Persistent):
         self._num_predecessors = self._ctr_predecessors = len(self._predecessor_ids)
 
 
+## -------------------------------------------------------------------------------------------------
+    def _raise_event(self, p_event_id: str, p_event_object: Event):
+        if p_event_id == self.C_EVENT_FINISHED: self._on_finished()
+        EventManager._raise_event(self, p_event_id, p_event_object)
+
+
+## -------------------------------------------------------------------------------------------------
+    def _on_finished(self):
+        """
+        Custom method that is called before an event C_EVENT_FINISHED is raised.
+        """
+        pass
+
+
 ## -------------------------------------------------------------------------------------------------
     def run_on_event(self, p_event_id, p_event_object:Event):
         """
@@ -732,7 +774,8 @@ class Task (Async, EventManager, Plottable, Persistent):
 ## -------------------------------------------------------------------------------------------------
     def init_plot( self, 
                    p_figure: Figure = None, 
-                   p_plot_settings : PlotSettings = None ):
+                   p_plot_settings : PlotSettings = None,
+                   p_window_title: str = None ):
         try:
             if ( not self.C_PLOT_ACTIVE ) or ( not self._visualize ): return
         except:
@@ -744,21 +787,21 @@ class Task (Async, EventManager, Plottable, Persistent):
             pass
 
         self.log(Log.C_LOG_TYPE_S, 'Init plot')
+
+        try:
+            view = p_plot_settings.view
+        except:
+            view = self.C_PLOT_DEFAULT_VIEW
+
+        if p_window_title is not None:
+            title = p_window_title
+        else:
+            title = 'MLPro: ' + self.C_TYPE + ' ' + self.get_name() + ' (' + view + ')'
+
         Plottable.init_plot( self,
                              p_figure=p_figure, 
-                             p_plot_settings=p_plot_settings )
-
-        if self._plot_own_figure:
-            title = 'MLPro: ' + self.C_TYPE + ' ' + self.get_name() + ' (' + self._plot_settings.view + ')'
-            backend = matplotlib.get_backend()
-
-            if backend == 'TkAgg':
-                self._figure.canvas.manager.window.title(title)
-            else:
-                try:
-                    self._figure.canvas.setWindowTitle(title)
-                except AttributeError:
-                    self._figure.canvas.set_window_title(title)
+                             p_plot_settings=p_plot_settings,
+                             p_window_title=title )
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -801,8 +844,6 @@ class Workflow (Task):
     C_TYPE          = 'Workflow'
     C_NAME          = ''
 
-    C_PLOT_ACTIVE   = True
-
 ## -------------------------------------------------------------------------------------------------
     def __init__( self, 
                   p_name:str=None,
@@ -827,7 +868,7 @@ class Workflow (Task):
                        p_class_shared=p_class_shared,
                        p_visualize=p_visualize,
                        p_logging=p_logging,
-                       p_kwargs=p_kwargs )
+                       **p_kwargs )
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -899,6 +940,17 @@ class Workflow (Task):
                 if t_pred in self._final_tasks: self._final_tasks.remove(t_pred)
 
 
+## -------------------------------------------------------------------------------------------------
+    def get_tasks(self) -> list:
+        return self._tasks
+
+
+## -------------------------------------------------------------------------------------------------
+    def reset(self, **p_kwargs):
+        for task in self._tasks:
+            task.reset(**p_kwargs)
+
+
 ## -------------------------------------------------------------------------------------------------
     def _get_plot_host_task(self, p_task : Task) -> Task:
         plot_host = None
@@ -917,7 +969,8 @@ class Workflow (Task):
 ## -------------------------------------------------------------------------------------------------
     def init_plot( self, 
                    p_figure:Figure=None, 
-                   p_plot_settings : PlotSettings = None ):
+                   p_plot_settings : PlotSettings = None,
+                   p_window_title: str = None ):
         """
         Initializes the plot of a workflow. The method creates a host figure for all tasks if no 
         external host figure is parameterized. The sub-plots of the tasks are autmatically arranged
@@ -931,63 +984,68 @@ class Workflow (Task):
             Optional MatPlotLib host figure, where the plot shall be embedded. The default is None.
         p_plot_settings : PlotSettings
             Optional plot settings. If None, the default view is plotted (see attribute C_PLOT_DEFAULT_VIEW).
+        p_window_title : str
+            Optional window title.
         """
 
-        try:
-            if ( not self.C_PLOT_ACTIVE ) or ( not self._visualize ): return
-        except:
-            return
-
+        # 1 Init plot output on workflow level (if activated)
         Task.init_plot( self,
-                        p_figure=p_figure, 
-                        p_plot_settings=p_plot_settings )
+                        p_figure = p_figure, 
+                        p_plot_settings = p_plot_settings,
+                        p_window_title = p_window_title )
+
       
+        # 2 Init plot output on task level
         for task in self._tasks:
-            task_plot_settings = []
 
-            if not task.C_PLOT_STANDALONE:
-                plot_host = self._get_plot_host_task(p_task=task)
-            else:
-                plot_host = self
+            task_figure        = None
+            task_plot_settings = p_plot_settings
 
-            ps = plot_host.get_plot_settings()
+            if task.get_visualization():
 
-            if task.C_PLOT_STANDALONE:
-                # Task plots in a separate figure (=window)
-                task_figure = None
-                task_axes   = None
-                task_pos_x  = 1
-                task_pos_y  = 1
-                task_ax_id  = 1
-                task_plot_settings = PlotSettings( p_view = ps.view,
-                                                   p_axes = task_axes,
-                                                   p_pos_x = task_pos_x,
-                                                   p_pos_y = task_pos_y,
-                                                   p_step_rate = ps.step_rate,
-                                                   p_plot_depth = ps.plot_depth,
-                                                   p_detail_level = ps.detail_level,
-                                                   p_force_fg = ps.force_fg,
-                                                   p_id=task_ax_id )
+                if not task.C_PLOT_STANDALONE:
+                    plot_host = self._get_plot_host_task(p_task=task)
+                else:
+                    plot_host = self
 
-            else:
-                # Task plots embedded in the predecessor/workflow figure/subplot
-                task_figure = plot_host._figure
-                task_plot_settings = ps
+                ps = plot_host.get_plot_settings()
+                if ps is None: ps = p_plot_settings
+
+                if task.C_PLOT_STANDALONE:
+                    # Task plots in a separate figure (=window)
+                    if ps is not None:
+                        task_plot_settings = ps.copy()
+                    else:
+                        task_plot_settings = PlotSettings( p_view = self.C_PLOT_DEFAULT_VIEW )
+
+                    task_plot_settings.axes  = None
+                    task_plot_settings.pos_x = 1
+                    task_plot_settings.pos_y = 1
+                    task_plot_settings.id    = 1
+
+                else:
+                    # Task plots embedded in the predecessor/workflow figure/subplot
+                    task_figure = plot_host._figure
+                    task_plot_settings = ps
                 
             task.init_plot( p_figure=task_figure,
                             p_plot_settings=task_plot_settings )
 
 
-        self.force_fg()
-        for task in self._tasks:
-            if task.C_PLOT_STANDALONE: task.force_fg()
-
-
-        if self._plot_own_figure:
+        # 3 Initial refresh of workflow window
+        if self.get_visualization() and self._plot_own_figure:
             self._figure.canvas.draw()
             self._figure.canvas.flush_events()
 
 
+## -------------------------------------------------------------------------------------------------
+    def remove_plot(self, p_refresh = True):
+        for task in self._tasks:
+            task.remove_plot( p_refresh=p_refresh )
+
+        return super().remove_plot(p_refresh)
+    
+
 ## -------------------------------------------------------------------------------------------------
     def run(self, p_range:int=None, p_wait:bool=False, **p_kwargs):
         """
