@@ -1,17 +1,19 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
-## -- Package : mlpro.rl.envs
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
+## -- Package : mlpro.bf.systems.pool
 ## -- Module  : doublependulum.py
 ## -------------------------------------------------------------------------------------------------
 ## -- History :
 ## -- yyyy-mm-dd  Ver.      Auth.    Description
-## -- 2023-03-05  0.0.0     LSB       Creation
-## -- 2023-03-05  1.0.0     LSB       Release
-## -- 2023-03-08  1.0.1     LSB       Refactoring for visualization
+## -- 2023-03-05  0.0.0     LSB      Creation
+## -- 2023-03-05  1.0.0     LSB      Release
+## -- 2023-03-08  1.0.1     LSB      Refactoring for visualization
+## -- 2024-12-11  1.0.2     DA       Refactoring      
+## -- 2025-07-18  1.1.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.0.1 (2023-03-08)
+Ver. 1.1.0 (2025-07-18)
 
 The Double Pendulum System is an implementation of a classic control problem of Double Pendulum system. The
 dynamics of the system are based on the `Double Pendulum <https://matplotlib.org/stable/gallery/animation/double_pendulum.html>`_  implementation by
@@ -21,15 +23,30 @@ Pendulum consists of an input motor providing the torque in either directions to
 """
 
 import random
+from collections import deque
+from datetime import timedelta
 
-from mlpro.rl import *
-from mlpro.bf.various import *
 import numpy as np
 from numpy import sin, cos
 from matplotlib.patches import Arc, RegularPolygon
 import scipy.integrate as integrate
-from collections import deque
 
+from mlpro.bf.various import *
+from mlpro.bf.exceptions import ParamError
+from mlpro.bf.events import *
+from mlpro.bf.mt import *
+from mlpro.bf.ops import Mode
+from mlpro.bf.plot import Figure, PlotSettings
+from mlpro.bf.math import ESpace, Dimension
+from mlpro.bf.streams import *
+from mlpro.bf.systems import *
+
+
+
+# Export list for public API
+__all__ = [ 'DoublePendulumSystemRoot',
+            'DoublePendulumSystemS4',
+            'DoublePendulumSystemS7' ]
 
 
 
@@ -123,8 +140,14 @@ class DoublePendulumSystemRoot (System):
 
 ## -------------------------------------------------------------------------------------------------
     def __init__ ( self,
+                   p_id = None,
+                   p_name : str =None,
+                   p_range_max : int = Async.C_RANGE_NONE,
+                   p_autorun = Task.C_AUTORUN_NONE,
+                   p_class_shared = None,
                    p_mode = Mode.C_MODE_SIM,
                    p_latency = None,
+                   p_t_step = None,
                    p_max_torque=20,
                    p_l1=1.0,
                    p_l2=1.0,
@@ -146,7 +169,8 @@ class DoublePendulumSystemRoot (System):
                    p_balancing_range:list = (-0.2,0.2),
                    p_swinging_outer_pole_range = (0.2,0.5),
                    p_break_swinging:bool = False,
-                   p_logging=Log.C_LOG_ALL ):
+                   p_logging=Log.C_LOG_ALL,
+                   **p_kwargs):
 
         self._max_torque = p_max_torque
 
@@ -172,7 +196,14 @@ class DoublePendulumSystemRoot (System):
 
 
         System.__init__(self,
+                        p_id = p_id,
+                        p_name = p_name,
+                        p_range_max = p_range_max,
+                        p_autorun = p_autorun,
+                        p_class_shared = p_class_shared,
                         p_mode=p_mode,
+                        p_latency=p_latency,
+                        p_t_step = p_t_step,
                         p_fct_strans= p_fct_strans,
                         p_fct_broken= p_fct_broken,
                         p_fct_success= p_fct_success,
@@ -183,7 +214,7 @@ class DoublePendulumSystemRoot (System):
                         p_camera_conf=p_camera_conf,
                         p_visualize=p_visualize,
                         p_logging=p_logging,
-                        p_latency=p_latency)
+                        **p_kwargs)
 
         self._t_step = self.get_latency().seconds + self.get_latency().microseconds / 1000000
 
@@ -485,15 +516,22 @@ class DoublePendulumSystemRoot (System):
                               theta2=250, color='crimson')
         endX = (0.5 * self._l1 / 2) * np.cos(np.radians(0))
         endY = (0.5 * self._l1 / 2) * np.sin(np.radians(0))
-        self._cw_arrow = RegularPolygon((endX, endY), 3, 0.5 * self._l1 / 9, np.radians(180),
-                                           color='crimson')
+        self._cw_arrow = RegularPolygon( xy = (endX, endY), 
+                                         numVertices = 3, 
+                                         radius = 0.5 * self._l1 / 9, 
+                                         orientation = np.radians(180),
+                                         color='crimson' )
 
         self._ccw_arc = Arc([0, 0], 0.5 * self._l1, 0.5 * self._l1, angle=70, theta1=0,
                                theta2=320, color='crimson')
         endX = (0.5 * self._l1 / 2) * np.cos(np.radians(70 + 320))
         endY = (0.5 * self._l1 / 2) * np.sin(np.radians(70 + 320))
-        self._ccw_arrow = RegularPolygon((endX, endY), 3, 0.5 * self._l1 / 9, np.radians(70 + 320),
-                                            color='crimson')
+
+        self._ccw_arrow = RegularPolygon( xy = (endX, endY), 
+                                          numVertices = 3, 
+                                          radius = 0.5 * self._l1 / 9, 
+                                          orientation = np.radians(70 + 320),
+                                          color='crimson' )
 
         p_settings.axes[0].add_patch(self._cw_arc)
         p_settings.axes[0].add_patch(self._cw_arrow)
@@ -510,7 +548,7 @@ class DoublePendulumSystemRoot (System):
 
 
 ## ------------------------------------------------------------------------------------------------------
-    def _update_plot_2d(self, p_settings: PlotSettings, **p_kwargs):
+    def _update_plot_2d(self, p_settings: PlotSettings, **p_kwargs) -> bool:
         """
         This method updates the plot figure of each episode. When the figure is
         detected to be an embedded figure, this method will only set up the
@@ -550,6 +588,8 @@ class DoublePendulumSystemRoot (System):
                     self._ccw_arrow.set_alpha(self._alpha)
         except: pass
 
+        return True
+
 
 
 
