@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.bf
 ## -- Module  : various
 ## -------------------------------------------------------------------------------------------------
@@ -45,10 +45,24 @@
 ## --                                - renamed class LoadSave to Persistent
 ## --                                - merged classes Load, Save into Persistent
 ## --                                - logging
+## -- 2023-04-12  2.1.1     MRD      Safe guarding open file with "with" 
+## -- 2023-06-01  2.1.2     SY       Scientific Referencing to bibtex format
+## -- 2024-05-19  2.2.0     DA       - class Id: new property id
+## --                                - class TStamp: new property tstamp
+## -- 2024-05-21  2.3.0     DA       Class TStamp: introduction of alias TStampType
+## -- 2024-06-18  2.4.0     DA       New class KWArgs
+## -- 2024-12-02  2.5.0     DA       New property KWargs.kwargs
+## -- 2024-12-06  2.6.0     DA       Class Log: tuning by about 10%
+## -- 2024-12-29  2.7.0     DA       - Method Log.log(): new parameter p_type_col
+## --                                - Class Log: code optimization
+## -- 2025-01-17  2.7.1     DA/SY    Correction of method Persistent.save()
+## -- 2025-03-11  2.7.2     SY       Enable recurse while pickling 
+## -- 2025-07-16  2.7.3     SY       Pickling desearilisation issue 
+## -- 2025-07-18  2.8.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 2.1.0 (2023-03-27)
+Ver. 2.8.0 (2025-07-18)
 
 This module provides various classes with elementry functionalities for reuse in higher level classes. 
 For example: logging, persistence, timer...
@@ -58,18 +72,34 @@ For example: logging, persistence, timer...
 from datetime import datetime, timedelta
 from time import sleep
 import dill as pkl
+import warnings
 import os
 import sys
 import uuid
+from typing import Union
+
 from mlpro.bf.exceptions import *
 
 
+
+# Export list for public API
+__all__ = [ 'Id',
+            'Log',
+            'Persistent',
+            'Timer',
+            'TStampType',
+            'ScientificObject',
+            'PersonalisedStamp',
+            'KWArgs' ]
+
+
+
+
 # Global dictionary to store paths of pickle files during runtime
 g_persistence_file_paths = {}
 
 
 
-
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
 class Id:
@@ -82,9 +112,9 @@ class Id:
         Optional external id
 
     Attributes
-    _id
+    ----------
+    id
         Unique id of the object.
-
     """
 
 ## -------------------------------------------------------------------------------------------------
@@ -114,6 +144,10 @@ class Id:
             self._id = uuid.uuid4()
 
 
+## -------------------------------------------------------------------------------------------------
+    id = property( fget=get_id, fset=set_id )
+
+
 
 
 
@@ -159,9 +193,14 @@ class Log:
 
 ## -------------------------------------------------------------------------------------------------
     def __init__(self, p_logging=C_LOG_ALL):
-        if p_logging not in self.C_LOG_LEVELS: 
-            raise ParamError('Wrong log level. See class Log for valid log levels')
-        self._level = p_logging
+
+        self._log_color_map = {
+            self.C_LOG_TYPE_W: self.C_COL_WARNING,
+            self.C_LOG_TYPE_E: self.C_COL_ERROR,
+            self.C_LOG_TYPE_S: self.C_COL_SUCCESS,
+        }
+
+        self._switch_logging( p_logging=p_logging )
 
         if self.C_INST_MSG:
             self.log(self.C_LOG_TYPE_I, 'Instantiated')
@@ -179,7 +218,7 @@ class Log:
 
 
 ## -------------------------------------------------------------------------------------------------
-    def switch_logging(self, p_logging):
+    def _switch_logging(self, p_logging):
         """
         Sets new log level. 
 
@@ -190,50 +229,67 @@ class Log:
 
         """
 
-        if p_logging not in self.C_LOG_LEVELS: raise ParamError('Wrong log level. See class Log for valid log levels')
+        if p_logging not in self.C_LOG_LEVELS: 
+            raise ParamError('Wrong log level. See class Log for valid log levels')
+        
         self._level = p_logging
+        if self._level:
+            self.log = self._log
+        else:
+            self.log = lambda *args, **kwargs: None
+
+
+ ## -------------------------------------------------------------------------------------------------
+    def switch_logging(self, p_logging):
+        self._switch_logging(p_logging=p_logging)
 
 
  ## -------------------------------------------------------------------------------------------------
     def get_log_level(self):
         return self._level
+    
 
- 
- ## -------------------------------------------------------------------------------------------------
-    def log(self, p_type, *p_args):
+## -------------------------------------------------------------------------------------------------
+    def log(self, p_type, *p_args, p_type_col = None):
         """
-        Writes log line to standard output in format:
-        yyyy-mm-dd  hh:mm:ss.mmmmmm  [p_type  C_TYPE C_NAME]: [p_args] 
+        Writes a log line to the standard output in the format:
+        yyyy-mm-dd  hh:mm:ss.mmmmmm  [p_type] [self.C_TYPE] [self.C_NAME]: [p_args] 
 
-        Parameters:
-            p_type      type of log entry
-            p_args      log informations
-
-        Returns: 
-            Nothing
+        Parameters
+        ----------
+        p_type
+            Type of log entry
+        p_args      
+            Optional log information
+        p_type_col = None
+            Optional differing log type used to color the log line.
         """
+        
+        # self.log is assigned dynamically to the method self._log. See constructor for more details...
+        pass
 
-        if not self._level: return
 
-        if self._level == self.C_LOG_WE:
-            if (p_type == self.C_LOG_TYPE_I) or (p_type == self.C_LOG_TYPE_S): return
-        elif self._level == self.C_LOG_E:
-            if (p_type == self.C_LOG_TYPE_I) or (p_type == self.C_LOG_TYPE_S) or (p_type == self.C_LOG_TYPE_W): return
+## -------------------------------------------------------------------------------------------------
+    def _log(self, p_type, *p_args, p_type_col = None):
 
-        now = datetime.now()
+        level = self._level
 
-        if p_type == self.C_LOG_TYPE_W:
-            col = self.C_COL_WARNING
-        elif p_type == self.C_LOG_TYPE_E:
-            col = self.C_COL_ERROR
-        elif p_type == self.C_LOG_TYPE_S:
-            col = self.C_COL_SUCCESS
+        if level == self.C_LOG_WE:
+            if p_type in (self.C_LOG_TYPE_I, self.C_LOG_TYPE_S): return
+        elif level == self.C_LOG_E:
+            if p_type in (self.C_LOG_TYPE_I, self.C_LOG_TYPE_S, self.C_LOG_TYPE_W): return
+            
+        now     = datetime.now().isoformat( sep = ' ' )
+
+        if p_type_col is not None:
+            type_col = p_type_col
         else:
-            col = self.C_COL_RESET
+            type_col = p_type
 
-        print(col + '%04d-%02d-%02d  %02d:%02d:%02d.%06d ' % (
-        now.year, now.month, now.day, now.hour, now.minute, now.second, now.microsecond),
-              p_type + '  ' + self.C_TYPE + ' "' + self.C_NAME + '":', *p_args, self.C_COL_RESET)
+        col     = self._log_color_map.get(type_col, self.C_COL_RESET)
+
+        arg_str = ' '.join(map(str, p_args))  
+        print( col + now + '  ' + p_type + '  ' + self.C_TYPE + ' "' + self.C_NAME + '": ' + arg_str + self.C_COL_RESET )
 
 
 
@@ -377,7 +433,8 @@ class Persistent (Id, Log):
 
         g_persistence_file_paths[p_filename] = p_path
 
-        obj = pkl.load(open(p_path + os.sep + p_filename, 'rb'))
+        with open(p_path + os.sep + p_filename, 'rb') as file:
+            obj = pkl.load(file)
 
         obj.log(Log.C_LOG_TYPE_I, 'Object loaded from file "' + p_path + os.sep + p_filename + '"')
 
@@ -459,19 +516,25 @@ class Persistent (Id, Log):
             True, if file content was saved successfully. False otherwise.
         """
 
+        pkl.settings["byref"] = True
+        warnings.filterwarnings("ignore", category=pkl.PicklingWarning)
+
         # 1 Create folder if it doesn't exist
         if not os.path.exists(p_path): os.makedirs(p_path)
 
         if p_filename is not None:
             filename = p_filename
+            self.set_filename( p_filename_stub=p_filename )
         else:
             filename = self.get_filename()
 
         g_persistence_file_paths[filename] = p_path
 
-        pkl.dump( obj=self, 
-                  file=open(p_path + os.sep + filename, "wb"),
-                  protocol=pkl.HIGHEST_PROTOCOL )
+        with open(p_path + os.sep + filename, "wb") as file:
+            pkl.dump( obj=self, 
+                    file=file,
+                    protocol=pkl.HIGHEST_PROTOCOL,
+                    recurse=True )
         
         self.log(Log.C_LOG_TYPE_I, 'Object saved to file "' + p_path + os.sep + filename + '"')
         return True
@@ -648,27 +711,39 @@ class Timer:
 
 
 
+TStampType = Union[ datetime, timedelta, float, int]
+
+
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
 class TStamp:
     """
-    This class provides elementry time stamp functionality for inherited classes.
+    This class provides elementry time stamp functionality for inherited classes. A time stamp in
+    MLPro can be neary every numeric type representing an absolute or relative time index. See
+    alias TStampType for further details.
+
+    Parameters
+    ----------
+    p_tstamp : TStampType
+        Optional initial time stamp.
     """
 
-
 ## -------------------------------------------------------------------------------------------------
-    def __init__(self, p_tstamp: timedelta = None):
+    def __init__(self, p_tstamp: TStampType = None):
         self.set_tstamp(p_tstamp)
 
 
 ## -------------------------------------------------------------------------------------------------
-    def get_tstamp(self) -> timedelta:
-        return self.tstamp
+    def get_tstamp(self) -> TStampType:
+        return self._tstamp
 
 
 ## -------------------------------------------------------------------------------------------------
-    def set_tstamp(self, p_tstamp: timedelta):
-        self.tstamp = p_tstamp
+    def set_tstamp(self, p_tstamp: TStampType):
+        self._tstamp = p_tstamp
+
+
+    tstamp = property( fget=get_tstamp, fset=set_tstamp )
 
 
 
@@ -682,10 +757,13 @@ class  ScientificObject:
     """
 
     C_SCIREF_TYPE_NONE = None
-    C_SCIREF_TYPE_ARTICLE = "Journal Article"
+    C_SCIREF_TYPE_ARTICLE = "Article"
     C_SCIREF_TYPE_BOOK = "Book"
+    C_SCIREF_TYPE_BOOKLET = "Booklet"
+    C_SCIREF_TYPE_INBOOK = "Inbook"
     C_SCIREF_TYPE_ONLINE = "Online"
     C_SCIREF_TYPE_PROCEEDINGS = "Proceedings"
+    C_SCIREF_TYPE_INPROCEEDINGS = "Inproceedings"
     C_SCIREF_TYPE_TECHREPORT = "Technical Report"
     C_SCIREF_TYPE_UNPUBLISHED = "Unpublished"
 
@@ -714,8 +792,100 @@ class  ScientificObject:
     C_SCIREF_CONFERENCE = None
     C_SCIREF_NOTES = None
     C_SCIREF_EDITOR = None
+    C_SCIREF_ADDRESS = None
+    C_SCIREF_HOWPUBLISHED = None
+    C_SCIREF_NUMPAGES= None
+    C_SCIREF_ISSN = None
+    C_SCIREF_VERSION = None
 
 
+## -------------------------------------------------------------------------------------------------
+    def get_bibtex(self):
+
+        if self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_ARTICLE:
+            field = '@article{CitekeyArticle,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_BOOK:
+            field = '@book{CitekeyBook,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_BOOKLET:
+            field = '@booklet{CitekeyBooklet,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_INBOOK:
+            field = '@inbook{CitekeyInbook,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_ONLINE:
+            field = '@misc{CitekeyMisc,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_PROCEEDINGS:
+            field = '@proceedings{CitekeyProceedings,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_INPROCEEDINGS:
+            field = '@inproceedings{CitekeyInproceedings,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_TECHREPORT:
+            field = '@techreport{CitekeyTechreport,'
+        elif self.C_SCIREF_TYPE == self.C_SCIREF_TYPE_UNPUBLISHED:
+            field = '@unpublished{CitekeyUnpublished,'
+        else:
+            raise ParamError('Type of the scientific reference is not defined!')
+        
+        content = '\n'
+        if self.C_SCIREF_AUTHOR is not None:
+            content += 'author = {%s},\n'%self.C_SCIREF_AUTHOR
+        if self.C_SCIREF_TITLE is not None:
+            content += 'title = {%s},\n'%self.C_SCIREF_TITLE
+        if self.C_SCIREF_JOURNAL is not None:
+            content += 'journal = {%s},\n'%self.C_SCIREF_JOURNAL
+        if self.C_SCIREF_ABSTRACT is not None:
+            content += 'abstract = {%s},\n'%self.C_SCIREF_ABSTRACT
+        if self.C_SCIREF_VOLUME is not None:
+            content += 'volume = {%s},\n'%self.C_SCIREF_VOLUME
+        if self.C_SCIREF_NUMBER is not None:
+            content += 'number = {%s},\n'%self.C_SCIREF_NUMBER
+        if self.C_SCIREF_PAGES is not None:
+            content += 'pages = {%s},\n'%self.C_SCIREF_PAGES
+        if self.C_SCIREF_YEAR is not None:
+            content += 'year = {%s},\n'%self.C_SCIREF_YEAR
+        if self.C_SCIREF_MONTH is not None:
+            content += 'month = {%s},\n'%self.C_SCIREF_MONTH
+        if self.C_SCIREF_DAY is not None:
+            content += 'day = {%s},\n'%self.C_SCIREF_DAY
+        if self.C_SCIREF_DOI is not None:
+            content += 'doi = {%s},\n'%self.C_SCIREF_DOI
+        if self.C_SCIREF_KEYWORDS is not None:
+            content += 'keywords = {%s},\n'%self.C_SCIREF_KEYWORDS
+        if self.C_SCIREF_ISBN is not None:
+            content += 'isbn = {%s},\n'%self.C_SCIREF_ISBN
+        if self.C_SCIREF_SERIES is not None:
+            content += 'series = {%s},\n'%self.C_SCIREF_SERIES
+        if self.C_SCIREF_PUBLISHER is not None:
+            content += 'publisher = {%s},\n'%self.C_SCIREF_PUBLISHER
+        if self.C_SCIREF_CITY is not None:
+            content += 'city = {%s},\n'%self.C_SCIREF_CITY
+        if self.C_SCIREF_COUNTRY is not None:
+            content += 'country = {%s},\n'%self.C_SCIREF_COUNTRY
+        if self.C_SCIREF_URL is not None:
+            content += 'url = {%s},\n'%self.C_SCIREF_URL
+        if self.C_SCIREF_CHAPTER is not None:
+            content += 'chapter = {%s},\n'%self.C_SCIREF_CHAPTER
+        if self.C_SCIREF_BOOKTITLE is not None:
+            content += 'booktitle = {%s},\n'%self.C_SCIREF_BOOKTITLE
+        if self.C_SCIREF_INSTITUTION is not None:
+            content += 'institution = {%s},\n'%self.C_SCIREF_INSTITUTION
+        if self.C_SCIREF_CONFERENCE is not None:
+            content += 'booktitle = {%s},\n'%self.C_SCIREF_CONFERENCE
+        if self.C_SCIREF_NOTES is not None:
+            content += 'note = {%s},\n'%self.C_SCIREF_NOTES
+        if self.C_SCIREF_EDITOR is not None:
+            content += 'editor = {%s},\n'%self.C_SCIREF_EDITOR
+        if self.C_SCIREF_ADDRESS is not None:
+            content += 'address = {%s},\n'%self.C_SCIREF_ADDRESS
+        if self.C_SCIREF_HOWPUBLISHED is not None:
+            content += 'howpublished = {%s},\n'%self.C_SCIREF_HOWPUBLISHED
+        if self.C_SCIREF_NUMPAGES is not None:
+            content += 'numpages = {%s},\n'%self.C_SCIREF_NUMPAGES
+        if self.C_SCIREF_ISSN is not None:
+            content += 'issn = {%s},\n'%self.C_SCIREF_ISSN
+        if self.C_SCIREF_VERSION is not None:
+            content += 'version = {%s},\n'%self.C_SCIREF_VERSION
+
+        print(field+content[:-2]+'\n}')
+
+        
 
 
 
@@ -741,6 +911,7 @@ class PersonalisedStamp (Id):
 
     C_NAME = ''
 
+
 ## -------------------------------------------------------------------------------------------------
     def __init__(self, p_name:str, p_id:int=None):
 
@@ -780,4 +951,71 @@ class PersonalisedStamp (Id):
             The unique name of the related component.
 
         """
-        return self._name
\ No newline at end of file
+        return self._name
+
+
+
+
+
+## -------------------------------------------------------------------------------------------------
+## -------------------------------------------------------------------------------------------------
+class KWArgs:
+    """
+    Enables standardized access to keyword parameters.
+
+    Parameters
+    ----------
+    p_kwargs : dict
+        Keyword parameters.
+    """
+
+## -------------------------------------------------------------------------------------------------
+    def __init__(self, **p_kwargs):
+        self._kwargs = p_kwargs.copy()
+
+
+## -------------------------------------------------------------------------------------------------
+    def _get_kwargs(self) -> dict:
+        """
+        Returns all keyword parameters.
+
+        Returns
+        -------
+        dict
+            Dictionary with keyword parameters.
+        """
+
+        return self._kwargs
+
+
+## -------------------------------------------------------------------------------------------------
+    def _get_kwarg(self, p_name: str, p_default = None):
+        """
+        Returns the value of a keyword parameter or its default value otherwise.
+
+        Parameters
+        ----------
+        p_name : str
+            Name of the keyword parameter.
+        p_default
+            Optional default value, if the parameter in p_name was not provided. An exception is
+            raised, if p_default is None and the keyword parameter was not provided.
+        
+        Returns
+        -------
+        any
+            The value of the keyword parameter if provided. Otherwise, p_default is returned as
+            a default value.
+        """
+
+        try:
+            return self._kwargs[p_name]
+        except:
+            if p_default is None:
+                raise ParamError('Keyword parameter "' + p_name + '" not supplied')
+            else:
+                return p_default
+            
+
+## -------------------------------------------------------------------------------------------------
+    kwargs = property( fget = _get_kwargs )
