@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.rl
 ## -- Module  : models_env.py
 ## -------------------------------------------------------------------------------------------------
@@ -43,19 +43,31 @@
 ## -- 2023-02-02  1.7.2     DA       All methods compute_reward(): unified name of first parameter 
 ## --                                to p_state_old
 ## -- 2023-02-13  1.7.3     MRD       Simplify State Space and Action Space generation
+## -- 2023-05-30  1.7.4     LSB      Redefining the inheritence order in EnvBase to resolve MRO in OAEnv
+## -- 2025-07-17  1.8.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.7.3 (2023-02-13)
+Ver. 1.8.0 (2025-07-17) 
 
 This module provides model classes for environments.
 """
 
+from datetime import timedelta
 
+from mlpro.bf import Log, Mode, ParamError
+from mlpro.bf.various import TStamp
 from mlpro.bf.systems import *
 
 
 
+# Export list for public API
+__all__ = [ 'Reward',
+            'FctReward',
+            'EnvBase',
+            'Environment' ]
+
+
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -244,7 +256,7 @@ class FctReward (Log):
 
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
-class EnvBase (System, FctReward):
+class EnvBase (FctReward, System):
     """
     Base class for all environment classes. It defines the interface and elementary properties for
     an environment in the context of reinforcement learning.
