@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.pool.policies
 ## -- Module  : randomgenerator
 ## -------------------------------------------------------------------------------------------------
@@ -11,26 +11,39 @@
 ## -- 2022-09-19  1.0.2     SY       Minor improvements: False operation for integers
 ## -- 2022-10-08  1.0.3     SY       Bug fixing
 ## -- 2022-11-02  1.0.4     DA       Refactoring: method _adapt()
+## -- 2023-09-21  1.0.5     SY       Typo on description
+## -- 2025-07-17  1.1.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.0.4 (2022-11-02)
+Ver. 1.1.0 (2025-07-17) 
 
-This module providew random genarator for multi purposes, e.g. testing environment, etc..
+This module provides random genarator for multi purposes, e.g. testing environment, etc..
 """
 
-from mlpro.rl.models import *
 import random
+
+import numpy as np
+
+from mlpro.bf import ParamError
+from mlpro.bf.systems import State, Action
+from mlpro.rl import Policy
          
         
         
+# Export list for public API
+__all__ = [ 'RandomGenerator' ]
+
+
+
+        
 ## -------------------------------------------------------------------------------------------------
 ## -------------------------------------------------------------------------------------------------
 class RandomGenerator (Policy):
     """
     A random policy that generates random actions for each dimension of the underlying action space.
 
-    See class mlpro.rl.Policy for furter details.
+    See class mlpro.rl.Policy for further details.
 
     """
 
