@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.sl.pool.FNN.pytorch
 ## -- Module  : mlp.py
 ## -------------------------------------------------------------------------------------------------
@@ -10,20 +10,36 @@
 ## -- 2023-03-10  1.1.0     SY       Combining _hyperparameters_check and _init_hyperparam
 ## -- 2023-03-28  1.2.0     SY       - Add _complete_state, _reduce_state due to new class Persistent
 ## --                                - Update _map
+## -- 2023-05-03  1.2.1     SY       Updating _adapt_offline method
+## -- 2023-06-21  1.2.2     LSB      Updating _adapt_offline method
+## -- 2023-07-04  1.2.3     LSB      Refactoring _complete_state for path conflict
+## -- 2023-07-14  1.2.4     LSB      Refactoring for afct fct parameter, so it is provided after instanciating
+## -- 2025-07-07  1.3.0     DA       Refactoring of method PyTorchMLP._map()
+## -- 2025-07-18  1.4.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.2.0 (2023-03-28)
+Ver. 1.4.0 (2025-07-18)
 
 This module provides a template ready-to-use MLP model using PyTorch. 
 """
 
+import numpy as np
+import torch
+
+from mlpro.bf import Log, ParamError
+from mlpro.bf.math import Element, MSpace
+from mlpro.bf.ml import HyperParam, HyperParamTuple
 
 from mlpro.sl.pool.afct.pytorch import *
-from mlpro.sl import *
+from mlpro.sl.fnn import MLP
 
 
 
+# Export list for public API
+__all__ = [ 'PyTorchMLP' ]
+
+
 
 
 ## -------------------------------------------------------------------------------------------------
@@ -115,15 +131,15 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
             if hd == 0:
                 act_input_size  = self.get_hyperparam().get_value(ids_[0])
                 output_size     = self.get_hyperparam().get_value(ids_[4])[hd]
-                act_fct         = self.get_hyperparam().get_value(ids_[5])[hd]()
+                act_fct         = self.get_hyperparam().get_value(ids_[5])[hd]
             elif hd == self.get_hyperparam().get_value(ids_[3]):
                 act_input_size  = self.get_hyperparam().get_value(ids_[4])[hd-1]
                 output_size     = self.get_hyperparam().get_value(ids_[1])
-                act_fct         = self.get_hyperparam().get_value(ids_[6])()
+                act_fct         = self.get_hyperparam().get_value(ids_[6])
             else:
                 act_input_size  = self.get_hyperparam().get_value(ids_[4])[hd-1]
                 output_size     = self.get_hyperparam().get_value(ids_[4])[hd]
-                act_fct         = self.get_hyperparam().get_value(ids_[5])[hd]()
+                act_fct         = self.get_hyperparam().get_value(ids_[5])[hd]
             
             if self.get_hyperparam().get_value(ids_[13]):
                 modules.append(init_(torch.nn.Linear(int(act_input_size), int(output_size))))
@@ -139,9 +155,10 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
         except:
             pass 
         
-        self._loss_fct = self.get_hyperparam().get_value(ids_[8])()
-        self._optimizer = self.get_hyperparam().get_value(ids_[7])(model.parameters(), lr=self.get_hyperparam().get_value(ids_[12]))
-
+        self._loss_fct      = self.get_hyperparam().get_value(ids_[8])()
+        self._optimizer     = self.get_hyperparam().get_value(ids_[7])(model.parameters(), lr=self.get_hyperparam().get_value(ids_[12]))
+        self._sampling_seed = self.get_hyperparam().get_value(ids_[11])
+        
         return model
     
 
@@ -320,7 +337,7 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
 
 
 ## -------------------------------------------------------------------------------------------------
-    def _map(self, p_input:Element, p_output:Element):
+    def _map(self, p_input:Element, p_output:Element, p_dim=None):
         """
         Maps a multivariate abscissa/input element to a multivariate ordinate/output element. 
 
@@ -346,6 +363,8 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
         # Set list to Element
         p_output.set_values(output)
 
+        return p_output
+
 
 ## -------------------------------------------------------------------------------------------------
     def _add_buffer(self, p_buffer_element:PyTorchIOElement):
@@ -423,15 +442,43 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
 
 
 ## -------------------------------------------------------------------------------------------------
-    def _adapt_offline(self, p_dataset) -> bool:
+    def _adapt_offline(self, p_dataset:dict) -> bool:
         """
         Adaptation mechanism for PyTorch based model for offline learning.
+
+        Parameters
+        ----------
+        p_dataset : dict
+            a dictionary that consists of a set of data, which are splitted to 2 keys such as input
+            and output. The value of each key is a torch.Tensor of the sampled data.
+
+        Returns
+        ----------
+            bool
         """
 
         self._sl_model.train()
-        outputs = self.forward(p_dataset["input"].dataset[0])
-        loss    = self._calc_loss(outputs, p_dataset["output"].dataset[0])
-        self._optimize(loss)
+        
+        for input, target in p_dataset:
+
+            try:
+                input = torch.tensor(input.get_values(), dtype=torch.float)
+                target = torch.tensor(target.get_values(), dtype=torch.float)
+            except:
+                pass
+
+            torch.manual_seed(self._sampling_seed)
+            outputs = self.forward(torch.squeeze(input))
+            
+            torch.manual_seed(self._sampling_seed)
+            self._loss    = self._calc_loss(outputs, torch.squeeze(target))
+
+            self._optimize(self._loss)
+
+            if isinstance(self._loss, torch.Tensor):
+                self._loss = self._loss.item()
+
+            self._sampling_seed += 1
             
         return True
 
@@ -455,7 +502,8 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
         BatchSize   = p_input.shape[0]
         output      = self._sl_model(p_input)   
         ids_        = self.get_hyperparam().get_dim_ids()
-        output      = output.reshape(BatchSize, int(self.get_hyperparam().get_value(ids_[1])))
+        # output      = output.reshape(BatchSize, int(self.get_hyperparam().get_value(ids_[1])))
+        
         return output
 
 
@@ -464,6 +512,8 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
         
         torch.save(p_state['_sl_model'].state_dict(),
                    p_path + p_os_sep + p_filename_stub + '_model.pt')
+        # print(p_state['_sl_model'].state_dict())
+
         torch.save(p_state['_optimizer'].state_dict(),
                    p_path + p_os_sep + p_filename_stub + '_optimizer.pt')
         
@@ -473,9 +523,15 @@ class PyTorchMLP (MLP, PyTorchHelperFunctions):
 
 ## -------------------------------------------------------------------------------------------------
     def _complete_state(self, p_path:str, p_os_sep:str, p_filename_stub:str):
-        
-        load_model      = torch.load(p_path + p_os_sep + 'model' + p_os_sep + p_filename_stub + '_model.pt')
-        load_optim      = torch.load(p_path + p_os_sep + 'model' + p_os_sep + p_filename_stub + '_optimizer.pt')
+        try:
+            load_model = torch.load(p_path + p_os_sep + p_filename_stub + '_model.pt')
+        except:
+            load_model      = torch.load(p_path + p_os_sep + 'model' + p_os_sep + p_filename_stub + '_model.pt')
+        # print(load_model)
+        try:
+            load_optim = torch.load(p_path + p_os_sep + p_filename_stub + '_optimizer.pt')
+        except:
+            load_optim      = torch.load(p_path + p_os_sep + 'model' + p_os_sep + p_filename_stub + '_optimizer.pt')
         self._sl_model  = self._setup_model()
         self._sl_model.load_state_dict(load_model)
         self._optimizer.load_state_dict(load_optim)
