@@ -1,5 +1,5 @@
 ## -------------------------------------------------------------------------------------------------
-## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
+## -- Project : MLPro - The integrative middleware framework for standardized machine learning
 ## -- Package : mlpro.sl.pool.afct
 ## -- Module  : pytorch.py
 ## -------------------------------------------------------------------------------------------------
@@ -14,21 +14,36 @@
 ## --                                 - Release the third version
 ## -- 2023-03-02  3.0.1     SY        Updating and shifting from mlpro.sl.models
 ## -- 2023-03-10  3.0.2     SY        Refactoring PyTorchBuffer
+## -- 2023-04-09  3.0.3     SY        Refactoring
+## -- 2023-05-03  3.0.4     SY        Updating sampling method
+## -- 2023-06-20  3.0.5     LSB       Updating the sampling method
+## -- 2023-07-02  3.0.6     LSB       Refactoring the postproc and preproc methods
+## -- 2023-07-14  3.0.7     LSB       Bug Fix
+## -- 2025-07-18  3.1.0     DA       Refactoring
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 3.0.2 (2023-03-10)
+Ver. 3.1.0 (2025-07-18)
 
 This a helper module for supervised learning models using PyTorch. 
 """
 
 
 import torch
+import numpy as np
 from torch.utils.data.sampler import SubsetRandomSampler
-from mlpro.sl.basics import *
-from mlpro.bf.ml import *
-from mlpro.bf.data import BufferElement, Buffer
 
+from mlpro.bf.data import BufferElement, Buffer
+from mlpro.bf.math import Element
+
+from mlpro.sl import *
+
+
+
+# Export list for public API
+__all__ = [ 'PyTorchIOElement',
+            'PyTorchBuffer',
+            'PyTorchHelperFunctions' ]
 
 
 
@@ -126,6 +141,17 @@ class PyTorchBuffer (Buffer, torch.utils.data.Dataset):
         """
         This method has a functionality to sample from the buffer using built-in PyTorch
         functionalities.
+
+        Returns
+        ----------
+        trainer : dict
+            a dictionary that consists of sampled data for training, which are splitted to 2 keys
+            such as input and output. The value of each key is a torch's DataLoader of the sampled
+            data.
+        tester : dict
+            a dictionary that consists of sampled data for testing, which are splitted to 2 keys
+            such as input and output. The value of each key is a torch's DataLoader of the sampled
+            data.
         """
 
         dataset_size    = len(self._data_buffer["input"])
@@ -139,23 +165,22 @@ class PyTorchBuffer (Buffer, torch.utils.data.Dataset):
         trainer         = {}
         tester          = {}
         
-        trainer["input"] = torch.utils.data.DataLoader(self._data_buffer["input"],
-                                                       batch_size=self._batch_size,
-                                                       sampler=train_sampler
-                                                       )
-        trainer["output"] = torch.utils.data.DataLoader(self._data_buffer["output"],
-                                                        batch_size=self._batch_size,
-                                                        sampler=train_sampler
-                                                        )
-        
-        tester["input"] = torch.utils.data.DataLoader(self._data_buffer["input"],
-                                                      batch_size=self._batch_size,
-                                                      sampler=test_sampler
-                                                      )
-        tester["output"] = torch.utils.data.DataLoader(self._data_buffer["output"],
-                                                       batch_size=self._batch_size,
-                                                       sampler=test_sampler
-                                                       )
+        trainer = torch.utils.data.DataLoader(self,
+                                              batch_size=self._batch_size,
+                                              sampler=train_sampler
+                                              )
+        # trainer["output"] = torch.utils.data.DataLoader(self._data_buffer["output"],
+        #                                                 batch_size=self._batch_size,
+        #                                                 sampler=train_sampler
+        #                                                 )
+        tester = torch.utils.data.DataLoader(self,
+                                             batch_size=self._batch_size,
+                                             sampler=test_sampler
+                                             )
+        # tester["output"] = torch.utils.data.DataLoader(self._data_buffer["output"],
+        #                                                batch_size=self._batch_size,
+        #                                                sampler=test_sampler
+        #                                                )
         
         return trainer, tester
     
@@ -188,7 +213,8 @@ class PyTorchHelperFunctions():
         """
 
         # Convert p_input from Element to Tensor
-        input = torch.Tensor([p_input.get_values()])
+        # input = torch.Tensor(np.array([p_input.get_values()]))
+        input = torch.Tensor(np.array(p_input.get_values()))
 
         # Preprocessing Data if needed
         try:
@@ -217,7 +243,7 @@ class PyTorchHelperFunctions():
         """
 
         # Convert p_output from Element to Tensor
-        output = torch.Tensor([p_output.get_values()])
+        output = torch.Tensor(np.array([p_output.get_values()]))
 
         # Preprocessing Data if needed
         try:
@@ -252,7 +278,7 @@ class PyTorchHelperFunctions():
             output = p_output
 
         # Convert output from Tensor to a list
-        output = output.detach().flatten().tolist()
+        output = output.detach().tolist()
 
         return output
 
