@@ -1,7 +1,7 @@
 ## -------------------------------------------------------------------------------------------------
 ## -- Project : MLPro - A Synoptic Framework for Standardized Machine Learning Tasks
 ## -- Package : mlpro
-## -- Module  : test_environment
+## -- Module  : test_environment.py
 ## -------------------------------------------------------------------------------------------------
 ## -- History :
 ## -- yyyy-mm-dd  Ver.      Auth.    Description
@@ -16,10 +16,11 @@
 ## -- 2022-09-02  1.0.5     SY       Add DoublePendulumS7 and DoublePendulumS4
 ## -- 2022-09-13  1.0.5     SY       Add Sim_MPPS
 ## -- 2022-11-22  1.0.6     SY       Remove Sim_MPPS
+## -- 2024-02-16  1.0.7     SY       Remove Multi-Cartpole
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 1.0.6 (2022-11-22)
+Ver. 1.0.7 (2024-02-16)
 
 Unit test classes for environment.
 """
@@ -28,19 +29,21 @@ Unit test classes for environment.
 import pytest
 import random
 import numpy as np
+
+from mlpro.bf.math import ESpace
+from mlpro.bf.systems import State, Action
+
 from mlpro.rl.models import *
 from mlpro.rl.pool.envs.robotinhtm import RobotHTM
 from mlpro.rl.pool.envs.bglp import BGLP
 from mlpro.rl.pool.envs.gridworld import GridWorld
-from mlpro.rl.pool.envs.multicartpole import MultiCartPole
-from mlpro.rl.pool.envs.doublependulum import DoublePendulumS7
-from mlpro.rl.pool.envs.doublependulum import DoublePendulumS4
+from mlpro.rl.pool.envs.doublependulum import DoublePendulumS4, DoublePendulumS7
 
 
 ## -------------------------------------------------------------------------------------------------
-@pytest.mark.parametrize("env_cls", [RobotHTM, BGLP, GridWorld, MultiCartPole, DoublePendulumS7, DoublePendulumS4])
+@pytest.mark.parametrize("env_cls", [RobotHTM, BGLP, GridWorld, DoublePendulumS7, DoublePendulumS4])
 def test_environment(env_cls):
-    env = env_cls()
+    env = env_cls(p_visualize=False)
     assert isinstance(env, Environment)
     
     assert isinstance(env.get_state_space(), ESpace)
